==> ubee512-1.2.0.tar.gz: ubee512-1.2.0/src/tape.c <==
//******************************************************************************
//*                                  uBee512                                   *
//*             An emulator for the Premium Microbee 512K FDD model            *
//*                                                                            *
//*                                Tape module                                 *
//*                                                                            *
//*                       Copyright (C) 2007 Stewart Kay                       *
//******************************************************************************
//
// This module is used to emulate the tape out and in circuit.
//
// tape out: data will be placed into a WAV file specified on the command line
//           with the --tapeo=filename option.
//
//  tape in: data will be read from a WAV file specified on the command line
//           with the --tapei=filename option.
//
//==============================================================================
// ChangeLog (most recent entries are at top)
//==============================================================================
// v1.2.0 - 20 August 2007, S.J.Kay
// - Created a new file and implement the tape emulation.

//==============================================================================
/*
 *  uBee512 - An emulator for the Premium Microbee 512K FDD model
 *  Copyright (C) 2007  Stewart Kay
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/ 
//==============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "function.h"
#include "tape.h"
#include "ubee512.h"
#include "mz80.h"

#include "macros.h"

char key_tape;

static wav_t wav_i;
static FILE *tape_i_file;
static uint64_t cycles_i_start;

static int tape_i_xtal;
static int tape_i_divval;
static int tape_i_off_cmp;
static int tape_i_datasize;
static int tape_i_channels;
static int tape_i_center;

extern char *tape_i;
extern char tapei[];

static wav_t wav_o;
static FILE *tape_o_file;
static uint64_t cycles_o_now;
static uint64_t cycles_o_before;
static uint64_t cycles_o_elapsed;
static int tape_o_rate;
static int tape_o_now;
static int tape_o_before;
static int tape_o_divval;
static int tape_o_off_cmp;
static int tape_o_size;

extern char *tape_o;
extern char tapeo[];

static int tape_o_level = 16;
static uint8_t buf[1024];

extern uint64_t z80_cycles;

//==============================================================================
// Tape input reset.
//
// For tape input the file is closed and then reopened,  the WAV file header
// is reloaded.
//
//   pass: void
// return: int				0 if no error, else -1
//==============================================================================
static int tape_i_reset (void)
{
 int error;
 char temp[5];
 
 if (tape_i != NULL)
    {
     if (tape_i_file != NULL)
        fclose(tape_i_file);        

     tape_i_file = fopen(tapei, "rb");
     if (tape_i_file == NULL)
        return -1;
     fread(&wav_i, sizeof(wav_i), 1, tape_i_file);  // read in the wave header
     tape_i_divval = tape_i_xtal / little_endian_u32(wav_i.sample_rate);
     tape_i_off_cmp = little_endian_u32(wav_i.sample_rate);
     tape_i_datasize = little_endian_u16(wav_i.bits_per_sample) / 8;
     tape_i_channels = little_endian_u16(wav_i.num_channels);
     tape_i_center = 0x01 << ((tape_i_datasize * 8) - 1);
     cycles_i_start = 0;
     key_tape = 0;

#if 0
     printf("tape_i_divval=%d  tape_i_off_cmp=%d  tape_i_datasize=%d tape_i_channels=%d tape_i_center=%d\n",
     tape_i_divval,  tape_i_off_cmp,  tape_i_datasize, tape_i_channels, tape_i_center);
#endif     

#if 1
     error = 0;
     temp[4] = 0;

     memcpy(temp, wav_i.chunk_id, 4);
     error |= strcmp(temp, "RIFF");
     
     memcpy(temp, wav_i.format, 4);     
     error |= strcmp(temp, "WAVE");
     
     memcpy(temp, wav_i.sub_chunk1_id, 4);
     error |= strcmp(temp, "fmt ");
     
     memcpy(temp, wav_i.sub_chunk2_id, 4);     
     error |= strcmp(temp, "data");

     error |= (wav_i.audio_format != 1);
     
     error |= ((tape_i_datasize == 0) | (tape_i_datasize > 4));
     
     error |= ((tape_i_datasize == 0) | (tape_i_datasize > 4));     
     
     if (error)
        {
         fclose(tape_i_file);
         printf("Unsupported wave file format\n");
         return -1;
        }
#endif
        
    }    
 return 0;
}

//==============================================================================
// Tape output reset.
//
//   pass: void
// return: int				0
//==============================================================================
static int tape_o_reset (void)
{
 return 0;
}

//==============================================================================
// Tape Initialise. 
//
// Open the tape input file and read in the WAV file header.
//
//   pass: void
// return: int				0 if success, -1 if error
//==============================================================================
int tape_init (void)
{
 if (tape_i != NULL)
    return tape_i_reset();

 return 0;
}

//==============================================================================
// Tape de-initialise.
//
// For tape out the WAV file header is filled in and the file closed.
// For tape in the file is closed.
//
//   pass: void
// return: int				0
//==============================================================================
int tape_deinit (void)
{
 if ((tape_o != NULL) && (tape_o_file != NULL))
    {
     memcpy(&wav_o.chunk_id, "RIFF", 4);     
     memcpy(&wav_o.format, "WAVE", 4);

     memcpy(&wav_o.sub_chunk1_id, "fmt ", 4);
     wav_o.sub_chunk1_size = little_endian_u32(16);
     wav_o.audio_format = little_endian_u16(1);		// PCM = 1 (i.e. Linear quantization)
     wav_o.num_channels = little_endian_u16(1);		// channels (mono)
     wav_o.sample_rate = little_endian_u32(tape_o_rate);  // sample rate
     wav_o.byte_rate = little_endian_u32(tape_o_rate);	// byte rate
     wav_o.block_align = little_endian_u16(1);
     wav_o.bits_per_sample = little_endian_u16(8);	// bits per sample

     memcpy(&wav_o.sub_chunk2_id, "data", 4);
     wav_o.sub_chunk2_size = big_endian_u32(tape_o_size);

     wav_o.chunk_size = little_endian_u32(36 + wav_o.sub_chunk2_size);     

     fseek(tape_o_file, 0, SEEK_SET);
     fwrite(&wav_o, sizeof(wav_o), 1, tape_o_file);  // write the wave file header
     fclose(tape_o_file);
    } 

 if (tape_i != NULL)
    fclose(tape_i_file);    

 return 0;
}

//==============================================================================
// Tape reset.
//
//   pass: void
// return: int				0
//==============================================================================
int tape_reset (void)
{
 return (tape_i_reset() | tape_o_reset());
}

//==============================================================================
// Tape read. Only call this if the tape input file is known to be open and
// key_tape > 0
//
// The WAV file will be expected to be in the format as defined for tape out
// usage.
// 
//   pass: void
// return: int				tape input bit value
//==============================================================================
int tape_r (void)
{
 int count;
 int filepos;
 uint32_t data;		// must be 32 bit
 int bytesread;

 uint64_t cycles_i_elapsed;
 uint64_t cycles_i_now; 

// pressing the F1 key will cause tape_i_state to be set to 2
 if (key_tape == 2)
    {
     tape_i_reset();
     key_tape = 1;
    } 

// calculate the elapsed time since the last read of the tape
// input,  this will be used to determine the amount to step into the WAV file
// to pull the next sample out.

 if (cycles_i_start == 0)
    {
     fseek(tape_i_file, sizeof(wav_i), SEEK_SET);
     if (! ferror(tape_i_file))
        {
         data = 0;  // clear all bytes in data
         bytesread = fread(&data, tape_i_datasize, 1, tape_i_file);  // read in one data sample
         data = little_endian_u32(data);     
         if (! bytesread)
            {
             tape_i_reset();
             return 0;
            }
         cycles_i_start = z80_cycles + mz80GetElapsedTicks(0); // use 0 !
        } 
    }
 else
    {               
     cycles_i_now = z80_cycles + mz80GetElapsedTicks(0); // use 0 !
     cycles_i_elapsed = (cycles_i_now - cycles_i_start);

// determine the sample offset we need
     count = (cycles_i_elapsed / tape_i_divval);

     filepos = sizeof(wav_i) + tape_i_datasize * tape_i_channels * count;
     fseek(tape_i_file, filepos, SEEK_SET);  // seek from start
     if (! ferror(tape_i_file))
        {
         data = 0;  // clear all bytes in data
         bytesread = fread(&data, tape_i_datasize, 1, tape_i_file);  // read in one data sample
         data = little_endian_u32(data);
         if (! bytesread)
            {
             tape_i_reset();
             return 0;
            }
        }    
    }
     if (data < tape_i_center)
        return B8(00000000);
     else
        return B8(00000001);
} 

//==============================================================================
// Tape write.  Only call this if the tape output file is known to be open.
//
//   pass: int data
// return: void
//==============================================================================
void tape_w (int data)
{
 int count;
 int tape;
 int x;

// mask of the tape out bit
 tape_o_now = data & B8(00000010);

// Only do something if the tape state changes.
 if (tape_o_now != tape_o_before)
    {
     tape_o_before = tape_o_now;

// Create the output wave file now if not already done so .  We do it here
// as it gives the user a chance to exit and not overwrite another file.
     if (tape_o != NULL)
        { 
         if (tape_o_file == NULL)
            {
             tape_o_size = 0;
             tape_o_file = fopen(tapeo, "wb");
             if (tape_o_file == NULL)
                return;
             memset(&wav_o, 0, sizeof(wav_o));
             fwrite(&wav_o, sizeof(wav_o), 1, tape_o_file);  // write an empty wave header
            }
        }     

// calculate the elapsed time since the last change of the tape
// output,  this will be used to determine the amount of data to be
// written out.
     cycles_o_now = z80_cycles + mz80GetElapsedTicks(0); // use 0 !
     cycles_o_elapsed = cycles_o_now - cycles_o_before;
     cycles_o_before = cycles_o_now;

// Determine the number of repeat values to be written out to the wave file.
     count = cycles_o_elapsed / tape_o_divval;

// We don't want a massive file so if long period cut it back shorter
// This is fine for normal KCS tape saves but not good for other things that
// use the tape port.  Will need an option to overide the behaviour.
     if (count > tape_o_off_cmp)
        {
         count = tape_o_off_cmp * 50;	// put out 5 seconds of silence
         tape = 128;			// silence
        }
     else

// get the actual fill value to be placed into the buffer.  The value used is
// the tape value on the previous occasion but it probably makes
// no difference anyway.
        {
         if (tape_o_before)
            tape = 128 + tape_o_level;
         else
            tape = 128 - tape_o_level;
        }    
        
     while (count)
        {
         if (count >= sizeof(buf))
            x = sizeof(buf);
         else
            x = count;
               
         memset(buf, tape, x);
         fwrite(buf, x, 1, tape_o_file);
         count -= x;
         tape_o_size += x;
        }
    }
}

//==============================================================================
// Tape output configuration.
//
// This determines the divider value to be used when creating the wave file
// data and data to placed into the wave header.
//
// The sample rate is the only parameter that can be changed for creating
// wave files.
//
// The output wave file format:
// mono, 8 bit data (128 is the center point), 
//
// The calcuations are always based on either a 3.375 or 2 MHz XTAL as these
// were the original speeds.  The correct application software is required
// for each XTAL frequency.  i.e. Old 2 MHz requires old BASIC.
//
//   pass: int xtal		XTAL frequency in Hz.
//         int rate		wave file sample rate
// return: void
//==============================================================================
void tape_config_out (int xtal, int rate)
{
 if ((xtal == 3375000) || (xtal == 2000000))
    tape_o_divval = xtal / rate;
 else   
    tape_o_divval = 3375000 / rate;    
 tape_o_off_cmp = rate / 10;
 tape_o_rate = rate;
}

//==============================================================================
// Tape input configuration.
//
// This saves the xtal value to be used later when loading a wave file
//
// The calcuations are always based on either a 3.375 or 2 MHz XTAL as these
// were the original speeds.  The correct application software is required
// for each XTAL frequency.  i.e. Old 2 MHz requires old BASIC.
//
//   pass: int xtal		XTAL frequency in Hz.
// return: void
//==============================================================================
void tape_config_in (int xtal)
{
 if ((xtal == 3375000) || (xtal == 2000000))
    tape_i_xtal = xtal;
 else   
    tape_i_xtal = 3375000;
}

==> ubee512-1.3.0.tar.gz: ubee512-1.3.0/src/tape.c <==
//******************************************************************************
//*                                  uBee512                                   *
//*             An emulator for the Premium Microbee 512K FDD model            *
//*                                                                            *
//*                                Tape module                                 *
//*                                                                            *
//*                       Copyright (C) 2007 Stewart Kay                       *
//******************************************************************************
//
// This module is used to emulate the tape out and in circuit.
//
// tape out: data will be placed into a WAV file specified on the command line
//           with the --tapeo=filename option.
//
//  tape in: data will be read from a WAV file specified on the command line
//           with the --tapei=filename option.
//
//==============================================================================
// ChangeLog (most recent entries are at top)
//==============================================================================
// v1.3.0 - 26 August 2007, S.J.Kay
// - Added tape volume level as a user settable option.
//
// v1.2.0 - 20 August 2007, S.J.Kay
// - Created a new file and implement the tape emulation.

//==============================================================================
/*
 *  uBee512 - An emulator for the Premium Microbee 512K FDD model
 *  Copyright (C) 2007  Stewart Kay
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/ 
//==============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "function.h"
#include "tape.h"
#include "ubee512.h"
#include "mz80.h"

#include "macros.h"

//==============================================================================
// structures and variables
//==============================================================================
char key_tape;

static wav_t wav_i;
static FILE *tape_i_file;
static uint64_t cycles_i_start;

static int tape_i_xtal;
static int tape_i_divval;
static int tape_i_off_cmp;
static int tape_i_datasize;
static int tape_i_channels;
static int tape_i_center;

extern char *tape_i;
extern char tapei[];

static wav_t wav_o;
static FILE *tape_o_file;
static uint64_t cycles_o_now;
static uint64_t cycles_o_before;
static uint64_t cycles_o_elapsed;
static int tape_o_rate;
static int tape_o_now;
static int tape_o_before;
static int tape_o_divval;
static int tape_o_off_cmp;
static int tape_o_size;

extern char *tape_o;
extern char tapeo[];

static int tape_o_level;
static uint8_t buf[1024];

extern uint64_t z80_cycles;

//==============================================================================
// Tape input reset.
//
// For tape input the file is closed and then reopened,  the WAV file header
// is reloaded.
//
//   pass: void
// return: int				0 if no error, else -1
//==============================================================================
static int tape_i_reset (void)
{
 int error;
 char temp[5];
 
 if (tape_i != NULL)
    {
     if (tape_i_file != NULL)
        fclose(tape_i_file);        

     tape_i_file = fopen(tapei, "rb");
     if (tape_i_file == NULL)
        return -1;
     fread(&wav_i, sizeof(wav_i), 1, tape_i_file);  // read in the wave header
     tape_i_divval = tape_i_xtal / little_endian_u32(wav_i.sample_rate);
     tape_i_off_cmp = little_endian_u32(wav_i.sample_rate);
     tape_i_datasize = little_endian_u16(wav_i.bits_per_sample) / 8;
     tape_i_channels = little_endian_u16(wav_i.num_channels);
     tape_i_center = 0x01 << ((tape_i_datasize * 8) - 1);
     cycles_i_start = 0;
     key_tape = 0;

#if 0
     printf("tape_i_divval=%d  tape_i_off_cmp=%d  tape_i_datasize=%d tape_i_channels=%d tape_i_center=%d\n",
     tape_i_divval,  tape_i_off_cmp,  tape_i_datasize, tape_i_channels, tape_i_center);
#endif     

     error = 0;
     temp[4] = 0;

     memcpy(temp, wav_i.chunk_id, 4);
     error |= strcmp(temp, "RIFF");
     
     memcpy(temp, wav_i.format, 4);     
     error |= strcmp(temp, "WAVE");
     
     memcpy(temp, wav_i.sub_chunk1_id, 4);
     error |= strcmp(temp, "fmt ");
     
     memcpy(temp, wav_i.sub_chunk2_id, 4);     
     error |= strcmp(temp, "data");

     error |= (wav_i.audio_format != 1);
     
     error |= ((tape_i_datasize == 0) | (tape_i_datasize > 4));
     
     error |= ((tape_i_datasize == 0) | (tape_i_datasize > 4));     
     
     if (error)
        {
         fclose(tape_i_file);
         printf("Unsupported wave file format\n");
         return -1;
        }
        
    }    
 return 0;
}

//==============================================================================
// Tape output reset.
//
//   pass: void
// return: int				0
//==============================================================================
static int tape_o_reset (void)
{
 return 0;
}

//==============================================================================
// Tape Initialise. 
//
// Open the tape input file and read in the WAV file header.
//
//   pass: void
// return: int				0 if success, -1 if error
//==============================================================================
int tape_init (void)
{
 if (tape_i != NULL)
    return tape_i_reset();

 return 0;
}

//==============================================================================
// Tape de-initialise.
//
// For tape out the WAV file header is filled in and the file closed.
// For tape in the file is closed.
//
//   pass: void
// return: int				0
//==============================================================================
int tape_deinit (void)
{
 if ((tape_o != NULL) && (tape_o_file != NULL))
    {
     memcpy(&wav_o.chunk_id, "RIFF", 4);     
     memcpy(&wav_o.format, "WAVE", 4);

     memcpy(&wav_o.sub_chunk1_id, "fmt ", 4);
     wav_o.sub_chunk1_size = little_endian_u32(16);
     wav_o.audio_format = little_endian_u16(1);		// PCM = 1 (i.e. Linear quantization)
     wav_o.num_channels = little_endian_u16(1);		// channels (mono)
     wav_o.sample_rate = little_endian_u32(tape_o_rate);  // sample rate
     wav_o.byte_rate = little_endian_u32(tape_o_rate);	// byte rate
     wav_o.block_align = little_endian_u16(1);
     wav_o.bits_per_sample = little_endian_u16(8);	// bits per sample

     memcpy(&wav_o.sub_chunk2_id, "data", 4);
     wav_o.sub_chunk2_size = big_endian_u32(tape_o_size);

     wav_o.chunk_size = little_endian_u32(36 + wav_o.sub_chunk2_size);     

     fseek(tape_o_file, 0, SEEK_SET);
     fwrite(&wav_o, sizeof(wav_o), 1, tape_o_file);  // write the wave file header
     fclose(tape_o_file);
    } 

 if (tape_i != NULL)
    fclose(tape_i_file);    

 return 0;
}

//==============================================================================
// Tape reset.
//
//   pass: void
// return: int				0
//==============================================================================
int tape_reset (void)
{
 return (tape_i_reset() | tape_o_reset());
}

//==============================================================================
// Tape read. Only call this if the tape input file is known to be open and
// key_tape > 0
//
// The WAV file will be expected to be in the format as defined for tape out
// usage.
// 
//   pass: void
// return: int				tape input bit value
//==============================================================================
int tape_r (void)
{
 int count;
 int filepos;
 uint32_t data;		// must be 32 bit
 int bytesread;

 uint64_t cycles_i_elapsed;
 uint64_t cycles_i_now; 

// pressing the F1 key will cause tape_i_state to be set to 2
 if (key_tape == 2)
    {
     tape_i_reset();
     key_tape = 1;
    } 

// calculate the elapsed time since the last read of the tape
// input,  this will be used to determine the amount to step into the WAV file
// to pull the next sample out.

 if (cycles_i_start == 0)
    {
     fseek(tape_i_file, sizeof(wav_i), SEEK_SET);
     if (! ferror(tape_i_file))
        {
         data = 0;  // clear all bytes in data
         bytesread = fread(&data, tape_i_datasize, 1, tape_i_file);  // read in one data sample
         data = little_endian_u32(data);     
         if (! bytesread)
            {
             tape_i_reset();
             return 0;
            }
         cycles_i_start = z80_cycles + mz80GetElapsedTicks(0); // use 0 !
        } 
    }
 else
    {               
     cycles_i_now = z80_cycles + mz80GetElapsedTicks(0); // use 0 !
     cycles_i_elapsed = (cycles_i_now - cycles_i_start);

// determine the sample offset we need
     count = (cycles_i_elapsed / tape_i_divval);

     filepos = sizeof(wav_i) + tape_i_datasize * tape_i_channels * count;
     fseek(tape_i_file, filepos, SEEK_SET);  // seek from start
     if (! ferror(tape_i_file))
        {
         data = 0;  // clear all bytes in data
         bytesread = fread(&data, tape_i_datasize, 1, tape_i_file);  // read in one data sample
         data = little_endian_u32(data);
         if (! bytesread)
            {
             tape_i_reset();
             return 0;
            }
        }    
    }
     if (data < tape_i_center)
        return B8(00000000);
     else
        return B8(00000001);
} 

//==============================================================================
// Tape write.  Only call this if the tape output file is known to be open.
//
//   pass: int data
// return: void
//==============================================================================
void tape_w (int data)
{
 int count;
 int tape;
 int x;

// mask of the tape out bit
 tape_o_now = data & B8(00000010);

// Only do something if the tape state changes.
 if (tape_o_now != tape_o_before)
    {
     tape_o_before = tape_o_now;

// Create the output wave file now if not already done so .  We do it here
// as it gives the user a chance to exit and not overwrite another file.
     if (tape_o != NULL)
        { 
         if (tape_o_file == NULL)
            {
             tape_o_size = 0;
             tape_o_file = fopen(tapeo, "wb");
             if (tape_o_file == NULL)
                return;
             memset(&wav_o, 0, sizeof(wav_o));
             fwrite(&wav_o, sizeof(wav_o), 1, tape_o_file);  // write an empty wave header
            }
        }     

// calculate the elapsed time since the last change of the tape
// output,  this will be used to determine the amount of data to be
// written out.
     cycles_o_now = z80_cycles + mz80GetElapsedTicks(0); // use 0 !
     cycles_o_elapsed = cycles_o_now - cycles_o_before;
     cycles_o_before = cycles_o_now;

// Determine the number of repeat values to be written out to the wave file.
     count = cycles_o_elapsed / tape_o_divval;

// We don't want a massive file so if long period cut it back shorter
// This is fine for normal KCS tape saves but not good for other things that
// use the tape port.  Will need an option to overide the behaviour.
     if (count > tape_o_off_cmp)
        {
         count = tape_o_off_cmp * 50;	// put out 5 seconds of silence
         tape = 128;			// silence
        }
     else

// get the actual fill value to be placed into the buffer.  The value used is
// the tape value on the previous occasion but it probably makes
// no difference anyway.
        {
         if (tape_o_before)
            tape = 128 + tape_o_level;
         else
            tape = 128 - tape_o_level;
        }    
        
     while (count)
        {
         if (count >= sizeof(buf))
            x = sizeof(buf);
         else
            x = count;
               
         memset(buf, tape, x);
         fwrite(buf, x, 1, tape_o_file);
         count -= x;
         tape_o_size += x;
        }
    }
}

//==============================================================================
// Tape output configuration.
//
// This determines the divider value to be used when creating the wave file
// data and data to placed into the wave header.
//
// The sample rate is the only parameter that can be changed for creating
// wave files.
//
// The output wave file format:
// mono, 8 bit data (128 is the center point), 
//
// The calcuations are always based on either a 3.375 or 2 MHz XTAL as these
// were the original speeds.  The correct application software is required
// for each XTAL frequency.  i.e. Old 2 MHz requires old BASIC.
//
//   pass: int xtal		XTAL frequency in Hz.
//         int rate		wave file sample rate
//         int volume		volume level
// return: void
//==============================================================================
void tape_config_out (int xtal, int rate, int volume)
{
 if ((xtal == 3375000) || (xtal == 2000000))
    tape_o_divval = xtal / rate;
 else   
    tape_o_divval = 3375000 / rate;    
 tape_o_off_cmp = rate / 10;
 tape_o_rate = rate;
 tape_o_level = volume;
}

//==============================================================================
// Tape input configuration.
//
// This saves the xtal value to be used later when loading a wave file
//
// The calcuations are always based on either a 3.375 or 2 MHz XTAL as these
// were the original speeds.  The correct application software is required
// for each XTAL frequency.  i.e. Old 2 MHz requires old BASIC.
//
//   pass: int xtal		XTAL frequency in Hz.
// return: void
//==============================================================================
void tape_config_in (int xtal)
{
 if ((xtal == 3375000) || (xtal == 2000000))
    tape_i_xtal = xtal;
 else   
    tape_i_xtal = 3375000;
}
